/*global define*/
/*jslint browser:true*/
define([
     'testsuite/qunit',
     'testsuite/qunit-fixture!de_epages/product/ui/variationslideshow-unity-test.html',
     'jquery',
     'de_epages/product/ui/variationslideshow-unity',
     'ep/core',

     '$ready!'
     ], function (QUnit, fixture, $, de_epages, ep) {
        'use strict';

    var imgString = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAIAAAACCAIAAAD91JpzAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAAJcEhZcwAADsMAAA7DAcdvqGQAAAARSURBVBhXYwCC////gzEDAwAp5AX7bk/yfwAAAABJRU5ErkJggg==",
        testUi = function(options) {
            // it is necessary to reference the element again (otherwise the widget is *undefined*)
            var widget = $('#OpenButton').data('uiProductUiVariationslideshowUnity');

            QUnit.equal(
                widget.templateElements.sortableList.children('li.ep-variationslideshow-listitem').length,
                options.dndSelected,
                options.dndSelected + ' image(s) in sortable List'
            );

            QUnit.equal(
                widget.templateElements.dndUploader.find('ul').children('li.selected').length,
                options.dndSelected,
                options.dndSelected + ' images on dndSorter marked as selected'
            );

            QUnit.equal(
                widget.templateElements.dndUploader.find('ul').children('li').length,
                options.dndAvailable,
                options.dndAvailable + ' images on dndSorter marked as selected'
            );
        },
        options = {
            items: function() {
                return [{
                    value: 1
                }];
            },
            dndUploader: {
                maxSize: 102400,
                exist: [{
                    "m_src": imgString,
                    "src": imgString,
                    "file": "img1.jpg",
                    "id": "-1"
                }, {
                    "m_src": imgString,
                    "src": imgString,
                    "file": "img2.jpg",
                    "id": "0"
                }, {
                    "m_src": imgString,
                    "src": imgString,
                    "file": "img3.jpg",
                    "id": "10"
                }, {
                    "m_src": imgString,
                    "src": imgString,
                    "file": "img4.jpg",
                    "id": "20"
                }, {
                    "m_src": imgString,
                    "src": imgString,
                    "file": "img5.jpg",
                    "id": "30"
                }],
                order: ["-1", "0", "10", "20", "30"],
            },
            action: 'changeAction',
            viewAction: 'viewAction',
            objectID: 9999,
            productName: 'ProductName',
            slideshowArrays: {
                1: ["img1.jpg","img2.jpg","img5.jpg"]
            }
        };

    QUnit.module("de_epages/product/ui/variationslideshow-unity'", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("widget test presence of methods", function () {
        fixture.init();

        $.each([
            '_create',
            '_openDialog',
            '_validateEnvironment',
            '_initializeSlideshows',
            '_initSorting',
            '_stopSorting',
            '_selectImageById',
            '_selectImageByName',
            '_selectImageByElement',
            '_removeImageById',
            '_removeImageByElement',
            '_handleSelectedListClick',
            '_handlePoolDrop',
            '_handlePoolClick',
            'destroy'
        ], function (i, fn) {
               QUnit.equal(
                $.isFunction($.ui.productUiVariationslideshowUnity.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        fixture.destroy();
        QUnit.start();
    });

    QUnit.asyncTest("test creating of ui from input data", function () {
        fixture.init();

        var $button = $('#OpenButton');

        de_epages('#OpenButton')
            .productUiVariationslideshowUnity(options);

        $('#OpenButton').data('uiProductUiVariationslideshowUnity')
            .templateElements.dndUploader.on('dnduploader:initialized', function () {
            var $openButton = $('#OpenButton'),
                $closeButton = $('.ui-dialog-titlebar-close'),
                widget = $('#OpenButton').data('uiProductUiVariationslideshowUnity');

            QUnit.equal(
                (widget !== undefined),
                true,
                'widget is resolved'
            );

            QUnit.equal(
                widget.dialog.uiDialog('isOpen'),
                true,
                'dialog is open'
            );

            QUnit.ok(true, "========= test with 2 products with same images =========");
            testUi({
                dndAvailable: 5,
                dndSelected: 3
            });

            $closeButton.trigger('click');

            QUnit.equal(
                widget.dialog.uiDialog('isOpen'),
                false,
                'dialog is closed'
            );

            de_epages('#OpenButton').productUiVariationslideshowUnity('destroy');
            fixture.destroy();
            QUnit.start();
        });
    });

    QUnit.asyncTest("test adding of image", function () {
        fixture.init();

        var $button = $('#OpenButton');

        de_epages('#OpenButton')
            .productUiVariationslideshowUnity(options);

        $('#OpenButton').data('uiProductUiVariationslideshowUnity')
            .templateElements.dndUploader.on('dnduploader:initialized', function () {
            var widget = $('#OpenButton').data('uiProductUiVariationslideshowUnity'),
                listItem;

            QUnit.equal(
                (widget !== undefined),
                true,
                'widget is resolved'
            );

            QUnit.equal(
                widget.dialog.uiDialog('isOpen'),
                true,
                'dialog is open'
            );

            QUnit.ok(true, "========= before adding image =========");

            testUi({
                dndAvailable: 5,
                dndSelected: 3
            });

            listItem = widget.templateElements.dndUploader.find('li:has(img[data-id="20"])');
            widget.templateElements.sortableList.append(listItem.clone());
            widget._handlePoolDrop('', {item: listItem});

            QUnit.ok(true, "========= after adding image =========");

            testUi({
                dndAvailable: 5,
                dndSelected: 4
            });

            widget.templateElements.dndUploader.find('li:has(img[data-id="10"])').trigger('click');

            QUnit.ok(true, "========= after adding 2nd image =========");

            testUi({
                dndAvailable: 5,
                dndSelected: 5
            });

            de_epages('#OpenButton').productUiVariationslideshowUnity('destroy');
            fixture.destroy();
            QUnit.start();
        });
    });

    QUnit.asyncTest("test uploading of image", function () {
        fixture.init();

        var $button = $('#OpenButton');

        de_epages('#OpenButton')
            .productUiVariationslideshowUnity(options);

        $('#OpenButton').data('uiProductUiVariationslideshowUnity')
            .templateElements.dndUploader.on('dnduploader:initialized', function () {
            var widget = $('#OpenButton').data('uiProductUiVariationslideshowUnity'),
                listItem;

            QUnit.equal(
                (widget !== undefined),
                true,
                'widget is resolved'
            );

            QUnit.equal(
                widget.dialog.uiDialog('isOpen'),
                true,
                'dialog is open'
            );

            QUnit.ok(true, "========= before upload completed event =========");

            testUi({
                dndAvailable: 5,
                dndSelected: 3
            });

            widget.templateElements.dndUploader.trigger('dnduploader:uploaded', {
                imageIds: [10]
            });

            QUnit.ok(true, "========= after upload completed event =========");

            testUi({
                dndAvailable: 5,
                dndSelected: 4
            });

            de_epages('#OpenButton').productUiVariationslideshowUnity('destroy');
            fixture.destroy();
            QUnit.start();
        });
    });

    QUnit.asyncTest("test deletion of image", function () {
        fixture.init();

        var $button = $('#OpenButton');

        de_epages('#OpenButton')
            .productUiVariationslideshowUnity(options);

        $('#OpenButton').data('uiProductUiVariationslideshowUnity')
            .templateElements.dndUploader.on('dnduploader:initialized', function () {
            var widget = $('#OpenButton').data('uiProductUiVariationslideshowUnity'),
                listItem;

            QUnit.equal(
                (widget !== undefined),
                true,
                'widget is resolved'
            );

            QUnit.equal(
                widget.dialog.uiDialog('isOpen'),
                true,
                'dialog is open'
            );

            QUnit.ok(true, "========= before deleting image =========");

            testUi({
                dndAvailable: 5,
                dndSelected: 3
            });

            widget.templateElements.sortableList.find('li:has(img[data-id="30"])').trigger('click');

            QUnit.ok(true, "========= after deleting image =========");

            testUi({
                dndAvailable: 5,
                dndSelected: 2
            });

            widget.templateElements.dndUploader.find('li:has(img[data-id="0"])').trigger('click');

            QUnit.ok(true, "========= after deleting 2nd image =========");

            testUi({
                dndAvailable: 5,
                dndSelected: 1
            });

            de_epages('#OpenButton').productUiVariationslideshowUnity('destroy');
            fixture.destroy();
            QUnit.start();
        });
    });

});