/*
 * @copyright		© Copyright 2006-2022, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.presentation.ui.confirmationdialog
 */

define("de_epages/presentation/ui/confirmationdialog", [
  "jquery",
  "ep",
  "$dict!ep/dict",

  "jquery/ui/draggable",
  "ep/fn/busy",
  "ep/ui/dialog"
], function ($, ep, epDict) {

/*
   * @dictionary      ep.dict
   *
   * @translation     {Cancel}
*/

  var confirmationdialog = {
    confirmationDialog: null,

    open: function (data) {
      if (!data.buttonActivatedSelector.hasClass('BatchActionDisabled')) {
        var dialogSelector = $(data.dialogClass);

        // create and append DOM node for dialog if not already present
        if (dialogSelector.length === 0) {
          dialogSelector = $('<div class="ep-product-portal-confirmationDialog container-fluid" />');
          $('body').append(dialogSelector);
        }

        // initialize confirmation dialog
        this.confirmationDialog = dialogSelector
          .html( '<p>' + data.text.DialogMessage + '</p>' )
          .uiDialog({
            title: data.text.DialogTitle,
            modal: true,
            autoOpen: false,
            draggable: true,
            resizable: false,
            width: 600,
            maxHeight: 600,
            buttons: [{
              cssClass: 'ep-button ep-button-save',
              text: data.text.DialogCommitButton,
              click: function () {
                confirmationdialog.submit(data);
                confirmationdialog.close();
              }
            }, {
              cssClass: 'ep-button ep-button-cancel',
              text: epDict.translate('{Cancel}'),
              click: function () {
                data.batchAction && data.batchAction.prop('checked', false);
                confirmationdialog.close();
              }
            }],
          });

        this.confirmationDialog.uiDialog('open');
      }
    },

    submit: function (data) {
      ep('body').busy('show');

      // execute onBeforeSubmit callback if necessary
      if (data.onBeforeSubmit && typeof data.onBeforeSubmit === "function") {
        data.onBeforeSubmit({
          form:  data.submitForm ? data.submitForm : undefined
        });
      }

      // submit form
      data.submitForm && data.submitForm.trigger('submit');
    },

    close: function () {
      this.confirmationDialog.uiDialog('close');
    }
  };

  return confirmationdialog;
});
